@@ -2,16 +2,66 @@ use strict;
 use v5.10.0;
 
 package FusqlFS::Backend::PgSQL::Functions;
+our $VERSION = "0.005";
 use parent 'FusqlFS::Artifact';
 
-use FusqlFS::Backend::PgSQL::Roles;
+=head1 NAME
 
-sub new
+FusqlFS::Backend::PgSQL::Functions - FusqlFS PostgreSQL database functions
+(a.k.a. stored procedures) interface
+
+=head1 SYNOPSIS
+
+    use FusqlFS::Backend::PgSQL::Functions;
+
+    my $funcs = FusqlFS::Backend::PgSQL::Functions->new();
+
+=head1 DESCRIPTION
+
+This is FusqlFS an interface to PostgreSQL database functions (or stored
+procedures as they are called in "big DBMS world"). This class is not to be
+used by itself.
+
+See L<FusqlFS::Artifact> for description of interface methods,
+L<FusqlFS::Backend> to learn more on backend initialization and
+L<FusqlFS::Backend::Base> for more info on database backends writing.
+
+=head1 EXPOSED STRUCTURE
+
+=over
+
+=item F<./language>
+
+Symlink to language used to write this function in F<../../languages>.
+See L<FusqlFS::Backend::PgSQL::Languages> for details.
+
+=item F<./content.*>
+
+This file have suffix equal to language name used to write this function and
+contains this function's body. You can edit it in order to change function's
+definition.
+
+=item F<./owner>
+
+Symlink to sequence's owner in F<../../roles>.
+
+=item F<./acl>
+
+Functions's ACL with permissions given to different roles. See
+L<FusqlFS::Backend::PgSQL::Role::Acl> for details.
+
+=back
+
+=cut
+
+use FusqlFS::Backend::PgSQL::Role::Owner;
+use FusqlFS::Backend::PgSQL::Role::Acl;
+
+sub init
 {
-    my $class = shift;
-    my $self = {};
+    my $self = shift;
 
-    my $pgver = $class->dbh->{pg_server_version};
+    my $pgver = $self->dbh->{pg_server_version};
     my $get_func_args = 'pg_catalog.pg_get_function_arguments(p.oid)';
     my $get_func_res  = 'pg_catalog.pg_get_function_result(p.oid)';
 
@@ -28,7 +78,7 @@ sub new
         END ||
         CASE
           WHEN COALESCE(p.proargnames[s.i], \'\') = \'\' THEN \'\'
-          ELSE p.proargnames[s.i] || \' \' 
+          ELSE p.proargnames[s.i] || \' \'
         END ||
         pg_catalog.format_type(p.proallargtypes[s.i], NULL)
       FROM
@@ -49,15 +99,15 @@ sub new
         $get_func_res = 'pg_catalog.format_type(p.prorettype, NULL)';
     }
 
-    $self->{list_expr} = $class->expr("SELECT DISTINCT p.proname||'('||$get_func_args||')' FROM pg_catalog.pg_proc AS p
+    $self->{list_expr} = $self->expr("SELECT DISTINCT p.proname||'('||$get_func_args||')' FROM pg_catalog.pg_proc AS p
                 LEFT JOIN pg_catalog.pg_namespace AS ns ON ns.oid = p.pronamespace
             WHERE ns.nspname = 'public'");
 
     #CASE WHEN p.proisagg THEN NULL ELSE pg_catalog.pg_get_functiondef(p.oid) END AS struct
-    $self->{get_expr} = $class->expr("SELECT $get_func_res AS result,
+    $self->{get_expr} = $self->expr("SELECT $get_func_res AS result,
                 trim(both from p.prosrc) AS content, l.lanname AS lang,
                 CASE p.provolatile WHEN 'i' THEN 'immutable' WHEN 's' THEN 'stable' WHEN 'v' THEN 'volatile' END AS volatility,
-                CASE WHEN p.proisagg THEN 'aggregate' WHEN p.proiswindow THEN 'window' WHEN p.prorettype = 'pg_catalog.trigger'::pg_catalog.regtype THEN 'trigger' ELSE NULL END AS type
+                CASE WHEN p.proisagg THEN 'aggregate' WHEN p.proiswindow THEN 'window' WHEN p.prorettype = 'pg_catalog.trigger'::pg_catalog.regtype THEN 'trigger' ELSE 'normal' END AS type
             FROM pg_catalog.pg_proc AS p
                 LEFT JOIN pg_catalog.pg_language AS l ON l.oid = p.prolang
             WHERE p.proname = ? AND $get_func_args = ?");
@@ -67,9 +117,8 @@ sub new
     $self->{drop_expr} = 'DROP FUNCTION %s';
     $self->{store_expr} = 'CREATE OR REPLACE FUNCTION %s RETURNS %s LANGUAGE %s AS $function$ %s $function$ %s';
 
-    $self->{owner} = new FusqlFS::Backend::PgSQL::Role::Owner('_F', 2);
-
-    bless $self, $class;
+    $self->{owner} = FusqlFS::Backend::PgSQL::Role::Owner->new('_F');
+    $self->{acl}   = FusqlFS::Backend::PgSQL::Role::Acl->new('_F');
 }
 
 =begin testing get
@@ -89,11 +138,12 @@ sub get
     return unless $data;
 
     my $result = {};
-    $result->{language} = \"../../languages/$data->{lang}";
+    $result->{language} = \"languages/$data->{lang}";
     $result->{'content.'.delete($data->{lang})} = delete($data->{content});
     $result->{struct} = $self->dump($data);
 
     $result->{owner} = $self->{owner};
+    $result->{acl}   = $self->{acl};
 
     return $result;
 }
@@ -139,22 +189,23 @@ sub store
 {
     my $self = shift;
     my ($name, $data) = @_;
-    my $struct = $self->load($data->{struct}||{});
-    return unless ref($struct) eq 'HASH';
+    return unless $data;
 
-    my ($contkey) = grep(/^content\./, keys %$data);
-    return unless $contkey;
-
-    my $content = $data->{$contkey};
-    my (undef, $lang) = split /\./, $contkey, 2;
-    return unless $content && $lang;
+    my $struct = $self->validate($data, {
+        struct => {
+            volatility => qr/^(volatile|immutable|stable)$/i,
+            type       => qr/^(window|trigger|normal)$/i,
+            result     => '',
+        },
+        language => ['SCALAR', sub{ $$_ =~ /^languages\/(.+)$/ && $1 }],
+    }, sub{ $_->{content} = $_[0]->{'content.'.$_->{language}}||undef })
+        or return;
 
     my $opts = ' ';
-    $opts .= 'WINDOW ' if defined($struct->{type}) && $struct->{type} eq 'window';
-    $opts .= uc $struct->{volatility} if defined($struct->{volatility})
-        and grep $struct->{volatility} eq $_, qw(volatile immutable stable);
+    $opts .= 'WINDOW ' if $struct->{struct}->{type} eq 'window';
+    $opts .= uc $struct->{struct}->{volatility};
 
-    $self->do($self->{store_expr}, [$name, $struct->{result}, $lang, $content, $opts]);
+    $self->do($self->{store_expr}, [$name, $struct->{struct}->{result}, $struct->{language}, $struct->{content}, $opts]);
 }
 
 =begin testing rename after create
@@ -211,24 +262,26 @@ __END__
 
 my $created_func = {
     'content.sql' => 'SELECT 1;',
-    'language' => \'../../languages/sql',
+    'language' => \'languages/sql',
     'struct' => '---
 result: integer
-type: ~
+type: normal
 volatility: volatile
 ',
     'owner' => $_tobj->{owner},
+    'acl' => $_tobj->{acl},
 };
 
 my $new_func = {
     'content.sql' => 'SELECT $1 | $2;',
-    'language' => \'../../languages/sql',
+    'language' => \'languages/sql',
     'struct' => '---
 result: integer
-type: ~
+type: normal
 volatility: immutable
 ',
     'owner' => $_tobj->{owner},
+    'acl' => $_tobj->{acl},
 };
 
 =end testing
