@@ -2,6 +2,7 @@ use strict;
 use v5.10.0;
 
 package FusqlFS::Backend::PgSQL::Table::Test;
+our $VERSION = "0.005";
 use FusqlFS::Backend::PgSQL::Test;
 
 our $fusqlh;
@@ -15,8 +16,8 @@ sub set_up
     $fusqlh->{subpackages}->{functions}->store('fusqlfs_function()',
         {
             'content.plperl' => 'return;',
-            struct => { result => 'trigger' },
-            language => \'../../languages/sql'
+            struct => { result => 'trigger', type => 'trigger', volatility => 'immutable' },
+            language => \'languages/plperl'
         }
     );
 }
